linux-kernel.vger.kernel.org archive mirror
 help / color / mirror / Atom feed
From: Pavel Tatashin <pasha.tatashin@soleen.com>
To: pasha.tatashin@soleen.com, jmorris@namei.org, sashal@kernel.org,
	ebiederm@xmission.com, kexec@lists.infradead.org,
	linux-kernel@vger.kernel.org, corbet@lwn.net,
	catalin.marinas@arm.com, will@kernel.org,
	linux-arm-kernel@lists.infradead.org, marc.zyngier@arm.com,
	james.morse@arm.com, vladimir.murzin@arm.com,
	matthias.bgg@gmail.com, bhsharma@redhat.com, linux-mm@kvack.org,
	mark.rutland@arm.com, steve.capper@arm.com, rfontana@redhat.com,
	tglx@linutronix.de
Subject: [PATCH v7 21/25] arm64: kexec: offset for relocation function
Date: Wed, 16 Oct 2019 16:00:30 -0400	[thread overview]
Message-ID: <20191016200034.1342308-22-pasha.tatashin@soleen.com> (raw)
In-Reply-To: <20191016200034.1342308-1-pasha.tatashin@soleen.com>

Soon, relocation function will share the same page with EL2 vectors.
Add offset within this page to arm64_relocate_new_kernel, and also
the total size of relocation code which will include both the function
and the EL2 vectors.

Signed-off-by: Pavel Tatashin <pasha.tatashin@soleen.com>
---
 arch/arm64/include/asm/kexec.h      |  7 +++++++
 arch/arm64/kernel/machine_kexec.c   | 13 ++++---------
 arch/arm64/kernel/relocate_kernel.S | 13 ++++++++-----
 3 files changed, 19 insertions(+), 14 deletions(-)

diff --git a/arch/arm64/include/asm/kexec.h b/arch/arm64/include/asm/kexec.h
index 189dce24f4cb..8cad34e7a9d9 100644
--- a/arch/arm64/include/asm/kexec.h
+++ b/arch/arm64/include/asm/kexec.h
@@ -90,6 +90,13 @@ static inline void crash_prepare_suspend(void) {}
 static inline void crash_post_resume(void) {}
 #endif
 
+#if defined(CONFIG_KEXEC_CORE)
+/* The beginning and size of relcation code to stage 2 kernel */
+extern const unsigned long kexec_relocate_code_size;
+extern const unsigned char kexec_relocate_code_start[];
+extern const unsigned long kexec_kern_reloc_offset;
+#endif
+
 /*
  * kern_reloc_arg is passed to kernel relocation function as an argument.
  * head		kimage->head, allows to traverse through relocation segments.
diff --git a/arch/arm64/kernel/machine_kexec.c b/arch/arm64/kernel/machine_kexec.c
index 5f1211f3aeef..5e7b1f6569c4 100644
--- a/arch/arm64/kernel/machine_kexec.c
+++ b/arch/arm64/kernel/machine_kexec.c
@@ -23,10 +23,6 @@
 
 #include "cpu-reset.h"
 
-/* Global variables for the arm64_relocate_new_kernel routine. */
-extern const unsigned char arm64_relocate_new_kernel[];
-extern const unsigned long arm64_relocate_new_kernel_size;
-
 /**
  * kexec_image_info - For debugging output.
  */
@@ -82,9 +78,8 @@ int machine_kexec_post_load(struct kimage *kimage)
 	if (!kern_reloc_arg)
 		return -ENOMEM;
 
-	memcpy(reloc_code, arm64_relocate_new_kernel,
-	       arm64_relocate_new_kernel_size);
-	kimage->arch.kern_reloc = __pa(reloc_code);
+	memcpy(reloc_code, kexec_relocate_code_start, kexec_relocate_code_size);
+	kimage->arch.kern_reloc = __pa(reloc_code) + kexec_kern_reloc_offset;
 	kimage->arch.kern_reloc_arg = __pa(kern_reloc_arg);
 	kern_reloc_arg->head = kimage->head;
 	kern_reloc_arg->entry_addr = kimage->start;
@@ -190,7 +185,7 @@ void machine_kexec(struct kimage *kimage)
 		"Some CPUs may be stale, kdump will be unreliable.\n");
 
 	/* Flush the reboot_code_buffer in preparation for its execution. */
-	__flush_dcache_area(reboot_code_buffer, arm64_relocate_new_kernel_size);
+	__flush_dcache_area(reboot_code_buffer, kexec_relocate_code_size);
 
 	/*
 	 * Although we've killed off the secondary CPUs, we don't update
@@ -199,7 +194,7 @@ void machine_kexec(struct kimage *kimage)
 	 * the offline CPUs. Therefore, we must use the __* variant here.
 	 */
 	__flush_icache_range((uintptr_t)reboot_code_buffer,
-			     arm64_relocate_new_kernel_size);
+			     kexec_relocate_code_size);
 
 	/* Flush the kimage list and its buffers. */
 	kexec_list_flush(kimage);
diff --git a/arch/arm64/kernel/relocate_kernel.S b/arch/arm64/kernel/relocate_kernel.S
index 22ccdcb106d3..3c05220a79ab 100644
--- a/arch/arm64/kernel/relocate_kernel.S
+++ b/arch/arm64/kernel/relocate_kernel.S
@@ -14,6 +14,8 @@
 #include <asm/page.h>
 #include <asm/sysreg.h>
 
+GLOBAL(kexec_relocate_code_start)
+
 /*
  * arm64_relocate_new_kernel - Put a 2nd stage image in place and boot it.
  *
@@ -86,13 +88,14 @@ ENTRY(arm64_relocate_new_kernel)
 .ltorg
 END(arm64_relocate_new_kernel)
 
-.Lcopy_end:
+.Lkexec_relocate_code_end:
 .org	KEXEC_CONTROL_PAGE_SIZE
 .align 3	/* To keep the 64-bit values below naturally aligned. */
 /*
  * arm64_relocate_new_kernel_size - Number of bytes to copy to the
  * control_code_page.
  */
-.globl arm64_relocate_new_kernel_size
-arm64_relocate_new_kernel_size:
-	.quad	.Lcopy_end - arm64_relocate_new_kernel
+GLOBAL(kexec_relocate_code_size)
+	.quad	.Lkexec_relocate_code_end - kexec_relocate_code_start
+GLOBAL(kexec_kern_reloc_offset)
+	.quad	arm64_relocate_new_kernel - kexec_relocate_code_start
@@ -86,13 +88,14 @@ ENTRY(arm64_relocate_new_kernel)
 .align 3	/* To keep the 64-bit values below naturally aligned. */
 /*
- * arm64_relocate_new_kernel_size - Number of bytes to copy to the
+ * kexec_relocate_code_size - Number of bytes to copy to the
  * control_code_page.
  */
-- 
2.23.0


  parent reply	other threads:[~2019-10-16 20:01 UTC|newest]

Thread overview: 27+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2019-10-16 20:00 [PATCH v7 00/25] arm64: MMU enabled kexec relocation Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 01/25] kexec: quiet down kexec reboot Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 02/25] kexec: add machine_kexec_post_load() Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 03/25] arm64: kexec: remove unnecessary debug prints Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 04/25] arm64: kexec: make dtb_mem always enabled Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 05/25] arm64: hibernate: pass the allocated pgdp to ttbr0 Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 06/25] arm64: hibernate: use get_safe_page directly Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 07/25] arm64: hibernate: remove gotos as they are not needed Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 08/25] arm64: hibernate: rename dst to page in create_safe_exec_page Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 09/25] arm64: hibernate: add PUD_SECT_RDONLY Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 10/25] arm64: hibernate: add trans_pgd public functions Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 11/25] arm64: hibernate: move page handling function to new trans_pgd.c Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 12/25] arm64: trans_pgd: make trans_pgd_map_page generic Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 13/25] arm64: trans_pgd: pass allocator trans_pgd_create_copy Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 14/25] arm64: trans_pgd: pass NULL instead of init_mm to *_populate functions Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 15/25] arm64: kexec: move relocation function setup Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 16/25] arm64: kexec: call kexec_image_info only once Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 17/25] arm64: kexec: cpu_soft_restart change argument types Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 18/25] arm64: kexec: arm64_relocate_new_kernel clean-ups Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 19/25] arm64: kexec: arm64_relocate_new_kernel don't use x0 as temp Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 20/25] arm64: kexec: add expandable argument to relocation function Pavel Tatashin
2019-10-16 20:00 ` Pavel Tatashin [this message]
2019-10-16 20:00 ` [PATCH v7 22/25] arm64: kexec: kexec EL2 vectors Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 23/25] arm64: kexec: configure trans_pgd page table for kexec Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 24/25] arm64: kexec: enable MMU during kexec relocation Pavel Tatashin
2019-10-16 20:00 ` [PATCH v7 25/25] arm64: kexec: remove head from relocation argument Pavel Tatashin
2019-10-17 14:26 ` [PATCH v7 00/25] arm64: MMU enabled kexec relocation Pavel Tatashin

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20191016200034.1342308-22-pasha.tatashin@soleen.com \
    --to=pasha.tatashin@soleen.com \
    --cc=bhsharma@redhat.com \
    --cc=catalin.marinas@arm.com \
    --cc=corbet@lwn.net \
    --cc=ebiederm@xmission.com \
    --cc=james.morse@arm.com \
    --cc=jmorris@namei.org \
    --cc=kexec@lists.infradead.org \
    --cc=linux-arm-kernel@lists.infradead.org \
    --cc=linux-kernel@vger.kernel.org \
    --cc=linux-mm@kvack.org \
    --cc=marc.zyngier@arm.com \
    --cc=mark.rutland@arm.com \
    --cc=matthias.bgg@gmail.com \
    --cc=rfontana@redhat.com \
    --cc=sashal@kernel.org \
    --cc=steve.capper@arm.com \
    --cc=tglx@linutronix.de \
    --cc=vladimir.murzin@arm.com \
    --cc=will@kernel.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).